// --- [ xbee_api_c.c ] ----------------------------------------------------------
//
// Ist fr das Senden via Frame0x08, Frame0x10 und Frame0x17 zustndig
//
// Framelnge: 0x08:  7 + Daten
//             0x10: 17 + Daten
//             0x17: 18 + Daten
//
// Ist fr das Empfangen von Frame 0x88, Frame0x8B, Frame0x92, Frame0x97 zustndig
//
// ber die serielle Schnittstelle eingehende Daten werden zunchst im rx_buffer[]
// abgelegt.
// Sind alle Bytes eines Frames empfangen, wird ein lokales rx_flag gesetzt
// Die Funktion FRM_is_received() liefert TRUE, wenn das flag gesetzt ist.
// Die Funktion FRM_get_frame_typ() liefert den Frame-Typ zurck.
// danach wird in Funktionen zur Auswertung der jeweiligen Frametypen verzweigt
//
// xbee_api -> Beim Empfangen via rs232 wird kein Timer exclusiv genutzt

#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <string.h>
#include "xbee_api_c.h"
#include "global.h"
#include "rs232_int.h"

#define START_DELIMITER    	0x7E

#define FRM_LENGTH          		2			// Bytepositionen innerhalb der
#define FRM_FRAME_TYP          	3			// verschiedenen Frames
#define FRM_FRAME_ID          	4        // nicht in allen Frames enthalten !!

#define FRM88_STATUS             7
#define FRM88_RESPONSE_START     8

#define FRM8A_STATUS					4

#define FRM8B_DELIVERY_STATUS    8
#define FRM8B_DISCOVERY_STATUS   9

#define FRM90_SOURCE_ADR			12
#define FRM90_RECEIVE_OPTIONS		14
#define FRM90_RX_DATA				15

#define FRM92_SOURCE_ADR         12
#define FRM92_RECEIVE_OPTIONS  	14
#define FRM92_SAMPLES         	15
#define FRM92_DIGITAL_CH_MASK  	16
#define FRM92_ANALOG_CH_MASK   	18
#define FRM92_SAMPLE_START       19

#define FRM97_STATUS             17
#define FRM97_RESPONSE_START     18

#define XBEE_RX_BUFFER_SIZE   	48			// hier muss das grte Frame reinpassen
#define CMD_BUFFER_SIZE          84       // hier muss das Kommando/Payload reinpassen

#define MSG_CTRL_SIZE	8						// 2^n !!!
#define MSG_CTRL_MASK   (MSG_CTRL_SIZE - 1)

// hier werden beim Versenden von Nachrichten der Index der Zieladresse und die Frame_Id eingetragen
unsigned char msg_ctrl[MSG_CTRL_SIZE][2];
unsigned char msg_ctrl_idx = 0;

uint8_t sender_idx = 255;
volatile uint8_t rx_flag;

// in diesem Array werden die zu sendenen Daten zusammengestellt
uint8_t cmd_buffer[CMD_BUFFER_SIZE];
uint8_t frame_id = 0;

uint8_t rx_buffer[XBEE_RX_BUFFER_SIZE];	// Empfangspuffer fr ein Frame
volatile uint8_t ISR_rx_idx = 0;				// der Index auf das nchste Byte im Frame

uint8_t API_checksum(void);
void FRM_to_air(uint8_t cnt);

uint8_t  FRM92_is_digital_sample(void);	// liefert zurck, ob digitale Samples aktiv

void API_check_64BitAdr(uint8_t * adr);
void API_check_16BitAdr(unsigned char id, uint8_t * adr);

// ### --- SENDEN --- ########################################################
//
// baut ein AT-Command auf und schreibt es in den Sendepuffer
//
// ---------------------------------------------------------------------------
// Parameter:
// - * cmd          : Array mit Daten
// - cmd_length     : Anzahl der zu sendenden Bytes
void FRM08_send_lokal_at_cmd(uint8_t * cmd, uint8_t cmd_length, uint8_t send_response)
{
uint8_t i = 0, j = 0;

cmd_buffer[i++] = START_DELIMITER;
cmd_buffer[i++] = 0;
i++;                                      // die Lnge berspringen
cmd_buffer[i++] = 0x08;                   // Frame Typ
if (!(++frame_id)) frame_id++;            // wenn Frame_ID == 0, dann Frame_ID = 1

if (send_response) 								// wenn send_response == TRUE,
	{
	cmd_buffer[i++] = frame_id;				// dann eine laufende Frame_ID senden
	}
else 													// wenn send_response == FALSE
	{
	cmd_buffer[i++] = 0;							// dann 0 als Frame_ID senden
	}
														// das Kommando incl. Parameter anhngen
while (cmd_length--)	cmd_buffer[i++] = cmd[j++];

cmd_buffer[2] = i - 3;                    // die Kommandolnge einfgen
cmd_buffer[i++] = API_checksum();         // die Prfsumme ins letzte Byte schreiben

FRM_to_air(i);                      		// schreibt das Frame in den Sendepuffer
}

// ----------------------------------------------------------------------------
// Parameter:
// - * cmd          : Array mit Daten
// - cmd_length     : Anzahl der zu sendenden Bytes
// - target_idx     : Target-Device, angesprochen ber seinen Index
// - send_id (bool) : TRUE  -> laufende Frame_ID senden, 
//                    FALSE -> Frame_ID = 0, keine Besttigung anfordern
void FRM10_send_data(uint8_t * data, uint8_t cmd_length, uint8_t target_idx, uint8_t send_response)
{
uint8_t i = 0, j = 0;

cmd_buffer[i++] = START_DELIMITER;
cmd_buffer[i++] = 0;
i++;                                      // die Lnge berspringen
cmd_buffer[i++] = 0x10;                   // Frame Typ

if (!(++frame_id)) frame_id++;            // wenn Frame_ID == 0, dann Frame_ID = 1

if (send_response) 								// wenn send_response == TRUE,
	{
	cmd_buffer[i++] = frame_id;				// dann eine laufende Frame_ID senden
	}
else 													// wenn send_response == FALSE
	{
	cmd_buffer[i++] = 0;							// dann 0 als Frame_ID senden
	}

for (j = 0; j < 10; j++)                  // 8 Byte Destination_Adr + 2 Byte
	{
	cmd_buffer[i++] = xbee_target_adr[target_idx][j];
	}

cmd_buffer[i++] = 0x0;                    // Broadcast Radius
cmd_buffer[i++] = 0x0;                    // Options

                                          // einen Eintrag ins Array msg_ctrl[] vornehmen
msg_ctrl_idx++;								   // den Index incrementieren
msg_ctrl_idx &= MSG_CTRL_MASK;			   // und begrenzen

msg_ctrl[msg_ctrl_idx] [0] = target_idx;	// Index des Ziels in target_adr[]
msg_ctrl[msg_ctrl_idx] [1] = frame_id;	   // die aktuelle Frameid

j = 0;
while (cmd_length--)	cmd_buffer[i++] = data[j++]; // Data anhngen

cmd_buffer[2] = i - 3;                    // die Kommandolnge einfgen
cmd_buffer[i++] = API_checksum();         // die Prfsumme ins letzte Byte schreiben

FRM_to_air(i);                      		// schreibt das Frame in den Sendepuffer
}


// ----------------------------------------------------------------------------
// Parameter:
// - * cmd          : Array mit Daten
// - cmd_length     : Anzahl der zu sendenden Bytes
// - target_idx     : Target-Device, angesprochen ber seinen Index
// - send_id (bool) : TRUE  -> laufende Frame_ID senden, 
//                    FALSE -> Frame_ID = 0, keine Besttigung anfordern
void FRM17_send_remote_at_cmd(uint8_t * cmd, uint8_t cmd_length, uint8_t target_idx, uint8_t send_response)
{
uint8_t i = 0, j = 0;

cmd_buffer[i++] = START_DELIMITER;
cmd_buffer[i++] = 0;
i++;                                      // die Lnge berspringen
cmd_buffer[i++] = 0x17;                   // Frame Typ

if (!(++frame_id)) frame_id++;            // wenn Frame_ID == 0, dann Frame_ID = 1

if (send_response) 								// wenn send_response == TRUE,
	{
	cmd_buffer[i++] = frame_id;				// dann eine laufende Frame_ID senden
	}
else 													// wenn send_response == FALSE
	{
	cmd_buffer[i++] = 0;							// dann 0 als Frame_ID senden
	}

for (j = 0; j < 10; j++)                  // 8 Byte Destination_Adr + 2 Byte
	{
	cmd_buffer[i++] = xbee_target_adr[target_idx][j];
	}

cmd_buffer[i++] = 0x02;                   // Apply Changes
                                          // einen Eintrag ins Array msg_ctrl[] vornehmen
msg_ctrl_idx++;								   // den Index incrementieren
msg_ctrl_idx &= MSG_CTRL_MASK;			   // und begrenzen

msg_ctrl[msg_ctrl_idx] [0] = target_idx;	// Index des Ziels in target_adr[]
msg_ctrl[msg_ctrl_idx] [1] = frame_id;	   // die aktuelle Frameid

j = 0;
while (cmd_length--)	cmd_buffer[i++] = cmd[j++]; // Kommando + Parameter anhngen

cmd_buffer[2] = i - 3;                    // die Kommandolnge einfgen
cmd_buffer[i++] = API_checksum();         // die Prfsumme ins letzte Byte schreiben

FRM_to_air(i);                      		// schreibt das Frame in den Sendepuffer
}

// ----------------------------------------------------------------------------
// addiert alle Bytes von Position 3 bis zum Byte vor der Prfsumme
// und subtrahiert das Ergebnis von 0xFF
uint8_t API_checksum(void)
{
uint8_t end, j, i = 0;

end = cmd_buffer[2] + (3 - 1);            // die Prfsumme nicht in die Prfsumme einschlieen

for (j = 3; j <= end; j++) i += cmd_buffer[j]; // die Prfsumme zhlt ab Index 3

return(0xFF - i);                         // den Wert von 255 abziehen
}

// ----------------------------------------------------------------------------
// Versendet ein fertig aufbereitetes Frame (das im Array cmd_buffer[] steht).
// es gibt unterschiedliche Methoden zum Senden der Daten
// - direktes Senden des Arrays
//   Vorteil: einfach zu implementieren
//   Nachteil: das Programm hngt lange in einer Schleife
// - das Array in einen Sendepuffer schreiben und via Interrupt die Bytes absenden
//   Vorteil: das Programm kann nach bergabe der Daten in die Hauptschleife zurck
//   Nachteil: bentigt Speicher fr einen Ringpuffer
//
void FRM_to_air(uint8_t cnt)
{
uint8_t i;
for (i = 0; i < cnt; i++) put_c(cmd_buffer[i]);
}

// ### --- EMPFANGEN --- ########################################################
//

// ----------------------------------------------------------------------------
// gibt zurck, ob ein Frame empfangen wurde
uint8_t FRM_is_received(void)
{
if (rx_flag & (1<<FRM_COMPLETE))
	{
	rx_flag &= ~(1<<FRM_COMPLETE);					// Flag lschen
	return(TRUE);
	}
return(FALSE);
}

// ----------------------------------------------------------------------------
// liefert den Frame Typ des Frames zurck, der steht immer an der selben Position
// eines Frames.
// Gleichzeitig wird die Prfsumme getestet.
// Bei fehlerhafter Prfsumme wird 0xFF zurckgeliefert,
// sonst der Frame_Type des Frames
// Ausserdem wird versucht, die von der Gegenstelle erhaltene 16-Bit Adresse
// in der eigenen Datenbank zu aktualisieren.
uint8_t FRM_get_frame_typ(void)
{
uint8_t i, end, frame, checksum = 0;     			// als 8 Bit-Wert

// --- 16-Bit Netzwerkadresse aktualisieren
frame = rx_buffer[FRM_FRAME_TYP];
if (frame == 0x8b)
{
   // Vergleicht die 16-Bit_Netzwerkadresse mit dem Eintrag in der Datenbank
   // der Index der Gegenstelle ist bekannt
   API_check_16BitAdr(rx_buffer[4], &rx_buffer[5]);
}
else if (frame == 0x97)
{
 	// Vergleicht die 16-Bit_Netzwerkadresse mit dem Eintrag in der Datenbank
 	// der Index der Gegenstelle ist bekannt
	API_check_16BitAdr(rx_buffer[4], &rx_buffer[13]);
}
else if ((frame == 0x90) ||(frame == 0x92) ||(frame == 0x95))
{
   // hier ist der Index der 64-Bit-Adresse nicht bekannt, daher zuerst
   // die 64-Bit-Adresse nachschlagen, dann die 16-Bit Netzwerkadresse prfen
   API_check_64BitAdr(&rx_buffer[4]);
}

end = rx_buffer[2] + 3;     // Lnge des Datensatzes bestimmen, Prfsumme einbeziehen
for (i = 3; i <= end; i++) checksum += rx_buffer[i];
if (checksum == 0xFF) return(frame);
else return(0xFF);
}

// --- FRAME 0x88 -------------------------------------------------------------
// ############################################################################
//
// --- Das Modem hat ein lokales AT-Kommando versandt und erhlt die Rckmeldung
//
// ----------------------------------------------------------------------------
// holt den Command Status von Frame0x88
// 0 =  OK, alles andere = Fehler
uint8_t FRM88_get_status(void)
{
return (rx_buffer[FRM88_STATUS]);
}

// ----------------------------------------------------------------------------
// Liefert zurck:
// - per Referenz die Rckmeldung auf das lokale AT-Kommando
// - per Value die Anzahl der Datenbytes in der Meldung
uint8_t FRM88_get_response(uint8_t * data)
{
uint8_t i = 0, j, end;

end = rx_buffer[FRM_LENGTH] + 3;
j   = FRM88_RESPONSE_START;

while (j < end) data[i++] = rx_buffer[j++];
return(i);
}

// --- FRAME 0x8A -------------------------------------------------------------
// ############################################################################
//
// --- Das Modem liefert unaufgefordert Meldungen zum eigenen Status
//
// ----------------------------------------------------------------------------
// liefert den Status aus dem Frame MODEM STATUS zurck
uint8_t FRM8A_get_status(void)
{
return(rx_buffer[FRM8A_STATUS]);
}

// --- FRAME 0x8B -------------------------------------------------------------
// ############################################################################
//
// --- Das Modem hat eine Nachricht versandt und liefert eine Rckmeldung
//
// ----------------------------------------------------------------------------
// gibt eine Meldung aus mit den Inhalt der Empfangsbesttigung
uint8_t FRM8B_get_delivery_status(void)
{
return(rx_buffer[FRM8B_DELIVERY_STATUS]);
}

// ----------------------------------------------------------------------------
// gibt eine Meldung aus mit den Inhalt der Empfangsbesttigung
uint8_t FRM8B_get_discovery_status(void)
{
return(rx_buffer[FRM8B_DISCOVERY_STATUS]);
}

// --- FRAME 0x90 -------------------------------------------------------------
// ############################################################################
//
// --- Das Modem hat eine Nachricht erhalten und liefert sie aus
//
// ----------------------------------------------------------------------------
// Liefert zurck:
// - per Referenz die 16 Bit Netzwerkadresse des antwortenden Modems
// - die Anzahl der Bytes in data[] - das sind aber immer 2 Byte !
/*
uint8_t FRM90_get_source_adr(uint8_t * data)
{
data[0] = rx_buffer[FRM90_SOURCE_ADR];
data[1] = rx_buffer[FRM90_SOURCE_ADR + 1];

return(2);
}
*/
// ----------------------------------------------------------------------------
// liefert das Receive-Options-Byte des Frames zurck:
// 1 = Acknowledged
// 2 = Broadcast
uint8_t FRM90_get_receive_options(void)
{
return(rx_buffer[FRM90_RECEIVE_OPTIONS]);
}

// ----------------------------------------------------------------------------
// Liefert zurck:
// - per Referenz die empfangenen Daten
// - per Value die Anzahl der empfangenen Bytes
uint8_t FRM90_get_data(uint8_t * data)
{
uint8_t i = 0, j, end;

end = rx_buffer[FRM_LENGTH] + 3;		// hier steht das letzte Datenbyte
j   = FRM90_RX_DATA;						// hier steht das erste  Datenbyte

while (j < end) data[i++] = rx_buffer[j++];
return(i);
}

// --- FRAME 0x92 -------------------------------------------------------------
// ############################################################################
//
// --- Das Modem hat eine IO-Statusmeldung von einem anderen Modem empfangen
//
// ----------------------------------------------------------------------------
// Liefert das Receive-Options-Byte des Frames zurck:
// 1 = Acknowledged
// 2 = Broadcast
uint8_t FRM92_get_receive_options(void)
{
return(rx_buffer[FRM92_RECEIVE_OPTIONS]);
}

/*
// ----------------------------------------------------------------------------
// Liefert zurck:
// - per Referenz die 16 Bit Netzwerkadresse des antwortenden Modems
// - die Anzahl der Bytes in data[] - das sind aber immer 2 Byte !

uint8_t FRM92_get_source_adr(uint8_t * data)
{
data[0] = rx_buffer[FRM92_SOURCE_ADR];
data[1] = rx_buffer[FRM92_SOURCE_ADR + 1];

return(2);
}
*/

// ----------------------------------------------------------------------------
// Prft die digitale IO-Maske:
// wenn kein Bit gesetzt ist, dann wird 0 zurckgeliefert, sonst ein Wert > 0.
// Ist der Wert grer als 0, dann ist mindestens 1 Pin als IO freigeschaltet.

uint8_t FRM92_is_digital_sample(void)
{
return (rx_buffer[FRM92_DIGITAL_CH_MASK] || rx_buffer[FRM92_DIGITAL_CH_MASK + 1]);
}


// ----------------------------------------------------------------------------
// gibt den Sample-Wert der unteren 8 Portpins zurck
// die oberen Pins werden hier ignoriert !
uint8_t FRM92_get_digital_sample_lowbyte(void)
{
return (rx_buffer[FRM92_DIGITAL_CH_MASK + 4]);
}

// ----------------------------------------------------------------------------
// liefert der Portstatus als String zurck - zur Vereinfachung hier nur implementiert
// fr die Ports D0 ... D7
//
// Ausgegeben wird in der Form: "_;_;1;0;_;_;1;0;"
// _ = nicht als digitaler Input definiert
// 0 = Pin auf low
// 1 = Pin auf high
// die Pins sind durch Semikolon getrennt
//
// der Status der digitalen Inputs wird in zwei 16Bit Variablen decodiert:
// - die erste Variable zeigt an, ob ein Pin als Input definiert ist (Maske)
//   (ACHTUNG: die Maske ist nicht identisch mit dem Wert, den IC liefert,
//    alle aktiven IO-Pins - auch wenn sie keinen Interrupt aufrufen - sind maskiert !)
// - die zweite Variable zeigt den Status der Pins an
// berholt ---- Die Anzahl der Bytes ist immer gleich 8 * 2 + \0

void FRM92_get_digital_inputs(char * txt)
{
uint8_t i, j, m ;
														// wenn kein Bit in der Maske gesetzt ist
														// dann gibt es keine digitalen Daten
//if (!(FRM92_digital_sample())) return(0);
/*
if (!(rx_buffer[FRM92_DIGITAL_CH_MASK] + rx_buffer[FRM92_DIGITAL_CH_MASK + 1])= return(0);
*/
j = 8;
i = 0;

while (j--)											// hier werden nur die unteren 8Bit getestet !!
   {
//	j--;
	m = ( 1<< j );									// Maske bilden
   if (rx_buffer[FRM92_DIGITAL_CH_MASK + 1] & m) // wenn der Pin als Input gesetzt
      {
      if (rx_buffer[FRM92_DIGITAL_CH_MASK + 4] & m)  // und der Pin auf High
         {
         txt[i++] = '1';						// Symbol fr eingeschaltet
         }

      else                                // Pin ist low
         {
         txt[i++] = '0';						// Symbol fr ausgeschaltet
         }
      }

   else                                   // Pin ist nicht als Input gesetzt
      {
      txt[i++] = '_';							// Zeichen fr deaktiviert
      }

   txt[i++] = SEPARATOR;                  // Trenner zwischen den Daten
	}

txt[i] = 0;											// die Zeichenfolge mit \0 abschlieen
														// damit die Pinmaske noch angehngt werden kann
utoa(rx_buffer[FRM92_DIGITAL_CH_MASK + 4], cv, 10);
strcat(txt, cv);

}


// ----------------------------------------------------------------------------
// schreibt die analogen Werte in dezimaler Form in einen String,
// die Datenstze sind durch Semikolon getrennt.
// wenn digitale Samples vorhanden sind, dann verschiebt sich das erste analoge Sample
// um 2 Byte nach hinten.
uint8_t FRM92_get_analog_inputs(char * txt)
{
uint8_t i, end;    								// wenn kein Bit in der Maske gesetzt ist
														// dann liegen keine analogen Samples vor
if ((rx_buffer[FRM92_ANALOG_CH_MASK]) == 0) return(0);
														// Die Samples beginnen ab Pos. 19
														// dort liegen 2 Byte fr digitale Daten
														// sofern digitale Daten gesampled werden
														// Die Position des ersten analogen Samples
														// ist abhngig davon, ob ein digitales Sample vorhanden
if (FRM92_is_digital_sample()) i = FRM92_SAMPLE_START + 2; else i = FRM92_SAMPLE_START;

														// jeweils 2 Byte bilden ein Sample
end = rx_buffer[FRM_LENGTH] + (2 - 1);		// das Ende des Frames beendet die Samples

txt[0] = 0;											// den String lschen
while(i <= end)
	{
   strcat((char *)txt, utoa( (rx_buffer[i] * 256 + rx_buffer[i+1]), cv, 10));
   strcat((char *)txt, ";");					// Samples durch Semikolon trennen
	i += 2;
   }
return(strlen((char*) txt));
}

// --- FRAME 0x97 -------------------------------------------------------------
// ############################################################################
//
// --- Das Modem hat ein Remote-AT-Kommando versandt und erhlt die Rckmeldung
//
// ----------------------------------------------------------------------------
// liefert den Command Status binr zurck.
// Nur '0' ist ein Erfolg.
uint8_t FRM97_get_status(void)
{
return(rx_buffer[FRM97_STATUS]);
}

// ----------------------------------------------------------------------------
// liefert zurck:
// - per Referenz die empfangenen Bytes im Array data[]
// - per Value    die Anzahl der empfangenen Bytes
uint8_t FRM97_get_response(uint8_t * data)
{
uint8_t i = 0, j, end;

end = rx_buffer[FRM_LENGTH] + 3;
j   = FRM97_RESPONSE_START;

while (j < end) data[i++] = rx_buffer[j++];
return(i);
}

// ----------------------------------------------------------------------------
//
// Diese Funktion wird aufgerufen, wenn eine Nachricht empfangen wird,
// die keine Antwort auf eine eigene Anfrage ist.
// Der Index der AbsenderAdr ist daher nicht im Array msg_ctrl[] eingetragen,
// er muss ber die 64-Bit-Adresse gesucht werden,
// erst dann kann die 16-Bit Adresse verglichen werden.
//
// Antworten auf Remote-AT und Data-Request werden hierrmit nicht behandelt,
// weil da der Absender (nmlich das Ziel versandten Nachricht) bekannt ist
// und daher nicht die 64-Bit-Adresse gesucht werden muss.
//
// bergeben wird eine Zeichenfolge bestehend aus device_adr und network_adr
// die ersten 8 Byte enthalten die device_adr, die folgenden 2 Byte die network_adr
// Das Programm durchsucht die Liste in target_adr[] und akutalisiert die
// zugehrige 16-Bit-Netzwerk_adr in target_network_adr
void API_check_64BitAdr(uint8_t * adr)
{
	unsigned char i, j = 0;
	unsigned char success;


// Index 0/1 sind reserviert fr Coordinator und Broadcast mit festen Adressen
	for (i = 2; i < (DEVICES); i++)
	{
		success = TRUE;

// Da die ersten 4 Byte der Adresse immer gleich sind, ist es effektiver,
// den Vergleich von hinten her aufzurollen:

		j = 7;
		while (j)
		{
			if (adr[j] != xbee_target_adr[i][j]) 
			{
				success = FALSE;
				break;
			}
			j--;
		}


		if (success)			// wenn kein Fehler beim Vergleich der 64-Bit-Adresse,
		{							// dann die 16-Bit-Adresse prfen

// ###  Einfach ohne Nachfrage die Netzwerkadresse berschreiben = aktualisieren
// wre mglich, aber dann kann natrlich keine Meldung ausgegeben werden !!!!!!

         xbee_target_adr[i][8] = adr[8];
         xbee_target_adr[i][9] = adr[9];
			sender_idx = i;
			return;
		}
		else sender_idx = 255;					// Fehlercode, wenn Adresse nicht gefunden wurde
   }
}

// ----------------------------------------------------------------------------
// Immer dann, wenn ein Remote-AT oder Data-Request versandt wird,
// wird die Anwort des Empfngers genutzt, um die 16-Bit-Netzwerkadresse zu prfen.
//
// Vor dem Senden des Frames wird die vergebene Frame_Id und der Index des Ziels im Array msg_ctrl[] abgelegt.
//
// Wenn dann spter die Antwort eingeht, dann wird nachgeschlagen, ob die 16-Bit-Netzwerkadresse,
// die der Empfnger zurckliefert, mit der bereinstimmt, die in der Target-Tabelle
// eingetragen ist
//
// benutzt die globalen Variablen msg_ctrl[] und msg_ctrl_idx
void API_check_16BitAdr(unsigned char frame_id, uint8_t * adr)
{
	unsigned char i, j, idx = 0;
	j = msg_ctrl_idx;						// die Suche an der letzten Position beginnen
	i = MSG_CTRL_SIZE;
	while (i--)
	{
		if (msg_ctrl[j][1] == frame_id)
		{
			idx = msg_ctrl[j][0];

			msg_ctrl[j][0] = 0;			// device_idx
			msg_ctrl[j][1] = 0;			// frame_id
			break;
		}
		j--;
		j &= MSG_CTRL_MASK;
	}

// wenn i == 0, dann ist die Suche nach der id erfolglos beblieben
	if (i)
	{
      xbee_target_adr[idx] [8] = adr[0];
		xbee_target_adr[idx] [9] = adr[1];
		sender_idx = idx;
   }
	else sender_idx = 255;
}

// ----------------------------------------------------------------------------
// Gibt den Index des Absenders der letzten Nachricht zurck
uint8_t FRM_get_sender_idx(void)
{
return(sender_idx);
}

// ----------------------------------------------------------------------------
// speichert ein Frame im Array rx_buffer[]
// sucht nach dem Start Delimiter und nachfolgendem 0
// dann wird das Frameende bestimmt

ISR (USART_RX_vect)
{
static uint8_t frame_end = 0;
register uint8_t rx_in, rx_idx;

rx_idx = ISR_rx_idx;								// volatile in Register einlesen
rx_in = UDR0;                             // empfangenes Byte

//rx_underflow = 2;									// um unvollstndige Sendungen zu erkennen

rx_buffer[rx_idx] = rx_in;           		// das Byte schon mal speichern
														// aber den Index erst spter incrementieren

if (rx_idx == 0)                         	// das Byte 0 muss 0x7E sein
	{
	if (!(rx_in == START_DELIMITER))	return;// wenn nicht, dann nichts tun
	}

else if (rx_idx == 1)                     // das Byte 1 muss 0 sein
	{
	if (rx_in)										// wenn Byte 1 doch grer als 0,
		{
		rx_idx = 0;									// dann beenden, dabei den Index zurcksetzen
		return;
		}
	}

else if (rx_idx == 2)                     // das Byte 2 ist die Lnge
	{
	frame_end = rx_in + 3;                 // Index des letzten Elementes
	}

else
	{
	if (rx_idx == frame_end)
		{
		rx_idx = 255;								// wird weiter unten durch incrementieren zu 0
		rx_flag = (1 << FRM_COMPLETE);			// rx_flag fr main() setzen
		TCCR2B = TIMER0_STOP;
//		rx_underflow = -1; 
		}
	}

ISR_rx_idx = ++rx_idx;							// Index incrementieren und speichern

// Mechanismus, um eine unvollstndige Sendung zu erkennen:
// (eine bertragung, in der die errechnete Anzahl von Bytes nicht zusammenkommt)
// Die bertragung luft mit 9600 Baud, also pro 1ms sollte ungefhr 1 Byte eingehen.
// Wenn lngere Zeit keine Daten kommen, dann ist die bertragung nicht wie erwartet.
// 
// erste Version: Timer starten, wenn der Interrupt zuschlgt, dann ist die Zeit
// abgelaufen, in der das nchste/letzte Byte eingetroffen sein sollte.

TCCR2A = 0;
TIMSK2 = (1<<TOV2);						      // wenn die bertragung nicht abgeschlossen wird
TCNT2  = TIMER2_PRELOAD;						// dann wird nach 2.2 ms in der
TCCR2B = TIMER2_START;							// dann wird in der ISR 'ISR_rx_idx' auf 0 gesetzt

}

// ----------------------------------------------------------------------------
// TIMER2_OVF:  3.686400 MHZ / (128 * 64) -> 2.2ms
// setzt ISR_rx_idx zurck, wenn mehr als 2.2 ms keine Aktivitt an der seriellen
// Schnittstelle stattfindet
ISR(TIMER2_OVF_vect)
{
TCCR2B = TIMER2_STOP;                     // Timer stoppen und den Index auf '0'
ISR_rx_idx = 0;									// das ist der Zweck der bung
}

// --- [ eof ] ----------------------------------------------------------------
